@@ -14,9 +14,6 @@
 #ifndef CEPH_MON_DATA_HEALTH_SERVICE_H
 #define CEPH_MON_DATA_HEALTH_SERVICE_H
 
-#include <boost/intrusive_ptr.hpp>
-// Because intusive_ptr clobbers our assert...
-#include "include/assert.h"
 #include <errno.h>
 
 #include "include/types.h"
@@ -66,9 +63,6 @@ public:
     set_update_period(g_conf->mon_health_data_update_interval);
   }
   virtual ~DataHealthService() { }
-  DataHealthService *get() {
-    return static_cast<DataHealthService *>(RefCountedObject::get());
-  }
 
   virtual void init() {
     generic_dout(20) << "data_health " << __func__ << dendl;
@@ -86,6 +80,5 @@ public:
     return "data_health";
   }
 };
-typedef boost::intrusive_ptr<DataHealthService> DataHealthServiceRef;
 
 #endif /* CEPH_MON_DATA_HEALTH_SERVICE_H */
