@@ -318,13 +318,22 @@ int MonClient::init()
 
 void MonClient::shutdown()
 {
+  ldout(cct, 10) << __func__ << "shutdown" << dendl;
   monc_lock.Lock();
   while (!version_requests.empty()) {
     version_requests.begin()->second->context->complete(-ECANCELED);
+    ldout(cct, 20) << __func__ << " canceling and discarding version request "
+		   << version_requests.begin()->second << dendl;
     delete version_requests.begin()->second;
     version_requests.erase(version_requests.begin());
   }
 
+  while (!waiting_for_session.empty()) {
+    ldout(cct, 20) << __func__ << " discarding pending message " << *waiting_for_session.front() << dendl;
+    waiting_for_session.front()->put();
+    waiting_for_session.pop_front();
+  }
+
   monc_lock.Unlock();
 
   if (initialized) {
@@ -516,6 +525,7 @@ void MonClient::_reopen_session(int rank, string name)
   // throw out version check requests
   while (!version_requests.empty()) {
     finisher.queue(version_requests.begin()->second->context, -EAGAIN);
+    delete version_requests.begin()->second;
     version_requests.erase(version_requests.begin());
   }
 
@@ -726,8 +736,6 @@ int MonClient::wait_auth_rotating(double timeout)
 
 void MonClient::_send_command(MonCommand *r)
 {
-  version_t last_seen_version = 0;
-
   if (r->target_rank >= 0 &&
       r->target_rank != monmap.get_rank(cur_mon)) {
     ldout(cct, 10) << "_send_command " << r->tid << " " << r->cmd
@@ -759,7 +767,7 @@ void MonClient::_send_command(MonCommand *r)
   }
 
   ldout(cct, 10) << "_send_command " << r->tid << " " << r->cmd << dendl;
-  MMonCommand *m = new MMonCommand(monmap.fsid, last_seen_version);
+  MMonCommand *m = new MMonCommand(monmap.fsid);
   m->set_tid(r->tid);
   m->cmd = r->cmd;
   m->set_data(r->inbl);
@@ -802,6 +810,23 @@ void MonClient::handle_mon_command_ack(MMonCommandAck *ack)
   ack->put();
 }
 
+int MonClient::_cancel_mon_command(uint64_t tid, int r)
+{
+  assert(monc_lock.is_locked());
+
+  map<tid_t, MonCommand*>::iterator it = mon_commands.find(tid);
+  if (it == mon_commands.end()) {
+    ldout(cct, 10) << __func__ << " tid " << tid << " dne" << dendl;
+    return -ENOENT;
+  }
+
+  ldout(cct, 10) << __func__ << " tid " << tid << dendl;
+
+  MonCommand *cmd = it->second;
+  _finish_command(cmd, -ETIMEDOUT, "");
+  return 0;
+}
+
 void MonClient::_finish_command(MonCommand *r, int ret, string rs)
 {
   ldout(cct, 10) << "_finish_command " << r->tid << " = " << ret << " " << rs << dendl;
@@ -826,6 +851,10 @@ int MonClient::start_mon_command(const vector<string>& cmd, bufferlist& inbl,
   r->poutbl = outbl;
   r->prs = outs;
   r->onfinish = onfinish;
+  if (cct->_conf->rados_mon_op_timeout > 0) {
+    r->ontimeout = new C_CancelMonCommand(r->tid, this);
+    timer.add_event_after(cct->_conf->rados_mon_op_timeout, r->ontimeout);
+  }
   mon_commands[r->tid] = r;
   _send_command(r);
   // can't fail
@@ -873,12 +902,13 @@ int MonClient::start_mon_command(int rank,
 
 void MonClient::get_version(string map, version_t *newest, version_t *oldest, Context *onfinish)
 {
-  ldout(cct, 10) << "get_version " << map << dendl;
+  version_req_d *req = new version_req_d(onfinish, newest, oldest);
+  ldout(cct, 10) << "get_version " << map << " req " << req << dendl;
   Mutex::Locker l(monc_lock);
   MMonGetVersion *m = new MMonGetVersion();
   m->what = map;
   m->handle = ++version_req_id;
-  version_requests[m->handle] = new version_req_d(onfinish, newest, oldest);
+  version_requests[m->handle] = req;
   _send_mon_message(m);
 }
 
@@ -887,10 +917,11 @@ void MonClient::handle_get_version_reply(MMonGetVersionReply* m)
   assert(monc_lock.is_locked());
   map<tid_t, version_req_d*>::iterator iter = version_requests.find(m->handle);
   if (iter == version_requests.end()) {
-    ldout(cct, 0) << "version request with handle " << m->handle
+    ldout(cct, 0) << __func__ << " version request with handle " << m->handle
 		  << " not found" << dendl;
   } else {
     version_req_d *req = iter->second;
+    ldout(cct, 10) << __func__ << " finishing " << req << " version " << m->version << dendl;
     version_requests.erase(iter);
     if (req->newest)
       *req->newest = m->version;
